'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useMailer = useMailer;

var _awsSdk = require('aws-sdk');

var _awsSdk2 = _interopRequireDefault(_awsSdk);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _nodemailer = require('nodemailer');

var _nodemailer2 = _interopRequireDefault(_nodemailer);

var _modes = require('./modes');

var _jwt = require('./jwt');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function useMailer(app, config) {
  var _this = this;

  // unpack
  var accountName = config.accountName,
      activationRoute = config.activationRoute,
      awsConfig = config.awsConfig,
      logger = config.logger,
      senderName = config.senderName,
      projectName = config.projectName,
      routePath = config.routePath,
      senderMail = config.senderMail;
  // use

  app.use(_bodyParser2.default.json());
  // nodemailer
  // use the ses smtp aws service via nodemailer transport
  // maybe use also just email-smtp.us-east-1.amazonaws.com
  _awsSdk2.default.config.update(awsConfig);
  var smtpTransport = _nodemailer2.default.createTransport({ SES: new _awsSdk2.default.SES({
      apiVersion: '2010-12-01'
    }) });
  // security, by default only the active users can do it
  var hasApiAccess = config.hasApiAccess;
  if (!hasApiAccess) {
    hasApiAccess = function hasApiAccess(req, res, next) {
      return (0, _jwt.hasModeApiAccess)(_modes.active, req, res, next, logger);
    };
  }
  // function
  function send(emails) {
    return Promise.all(emails.map(function (email) {
      return new Promise(function (resolve, reject) {
        /*
        if (!IS_PROD) {
          logger.debug('DEV: would have sent email to', email.to, '(not sending because: !IS_PROD)', email.subject, '\n', email.html)
          resolve()
          return
        }
        */
        if (!smtpTransport) {
          reject('Email smtpTransport not initiated, cannot send email');
          return;
        }
        smtpTransport.sendMail(email, function (error, info) {
          if (error) {
            logger.warn('sendEmail error', email, error);
            reject(error);
            return;
          }
          logger.debug('Email message sent: ' + info);
          resolve();
        });
      });
    }));
  }
  // api
  if (routePath) {
    app.post(routePath, hasApiAccess, function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(req, res) {
        var _req$body, email, emails, info;

        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _req$body = req.body, email = _req$body.email, emails = _req$body.emails;

                if (!emails) {
                  emails = [email];
                }
                _context.next = 4;
                return send(emails);

              case 4:
                info = _context.sent;

                res.send({ info: info });

              case 6:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this);
      }));

      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }());
  }
  // return
  return Object.assign({ send: send }, config);
}