'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFormatDocument = getFormatDocument;
exports.useSignin = useSignin;

var _bcryptNodejs = require('bcrypt-nodejs');

var _bcryptNodejs2 = _interopRequireDefault(_bcryptNodejs);

var _helmet = require('helmet');

var _helmet2 = _interopRequireDefault(_helmet);

var _passport = require('passport');

var _passport2 = _interopRequireDefault(_passport);

var _passportLocal = require('passport-local');

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NO_MATCHING_USER = 'NO_MATCHING_USER';
var INCORRECT_USER_CREDENTIALS = 'INCORRECT_USER_CREDENTIALS';

// from: http://stackoverflow.com/a/46181
// NOTE: is it good enough?
/* eslint-disable */
var VALID_EMAIL_REGEXP = /^(([^<>()\[\]\.,;:\s@\"]+(\.[^<>()\[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i;
var MIN_PASSWORD_LENGTH = 6;
var REQUIRED_SIGNUP_FIELDS = ['email', 'password'];
var globalUserFields = [];

function formatUserForFrontend(user) {
  // create a modified user object to return to frontend
  // NOTE: we assume it was a local user sign in, i.e. not via oauth
  var userCopy = Object.assign({}, user.local, {
    id: user.id,
    active: user.active,
    admin: user.admin,
    consoleToken: user.consoleToken,
    vat: user.vat,
    firstname: user.firstname,
    lastname: user.lastname,
    company: user.company
  });
  delete userCopy.password;
  delete userCopy.passwordResetToken;
  return userCopy;
}

function getFormatDocument(document, collectionName) {
  if (collectionName === 'users') {
    return formatUserForFrontend(document);
  }
  return document;
}

function signinUser(email, password, usersCollection) {
  return new Promise(function (resolve, reject) {
    // try to find matching user and password
    usersCollection.findOne({ 'local.email': email }).then(function (user) {
      if (!user) {
        reject(new Error(NO_MATCHING_USER));
        return;
      }
      return validatePassword(password, user.local.password).then(function (validated) {
        if (!validated) {
          reject(new Error(INCORRECT_USER_CREDENTIALS));
          return;
        }
        // correct match, return user
        resolve(user);
      });
    });
  });
}

function validatePassword(password, encryptedPassword) {
  return new Promise(function (resolve, reject) {
    if (password && encryptedPassword) {
      _bcryptNodejs2.default.compare(password, encryptedPassword, function (err, result) {
        if (err) {
          reject(err);
          return;
        }
        return resolve(result);
      });
    } else {
      reject('password or encryptedPassword not good in validatePassword', password, encryptedPassword);
    }
  });
}

function useSignin(app, config) {
  // unpack
  var db = config.db,
      logger = config.logger,
      routePath = config.routePath;

  var endpoint = config.endpoint || 'signin';
  var bodyKey = endpoint + 'Body';
  var messageKey = endpoint + 'Message';
  var passportKey = 'local-' + endpoint;
  // users
  var usersCollection = db.collection('users');
  // strategy
  var passportLocalStrategy = new _passportLocal.Strategy({
    usernameField: 'email',
    passReqToCallback: true // get access to full request in callback
  }, function (req, email, password, done) {
    // store the pushed data so we can keep the form populated in case of failure
    req.flash(bodyKey, req.body);
    signinUser(email, password, usersCollection).then(function (user) {
      return done(null, user);
    }).catch(function (err) {
      if (err.message === NO_MATCHING_USER) {
        done(null, false, req.flash(messageKey, 'No matching user found.'));
        return;
      }
      if (err.message === INCORRECT_USER_CREDENTIALS) {
        done(null, false, req.flash(messageKey, 'Incorrect credentials.'));
        return;
      }
      // TODO: handle this error!
      logger.warn('[unhandled] ERROR in handleLogin:', err);
      done(null, false, req.flash(messageKey, 'Login failed.'));
    });
  });
  _passport2.default.use(passportKey, passportLocalStrategy);
  // apis
  function returnTo(req, res, next) {
    if (req.body.returnTo) {
      req.session.returnTo = decodeURIComponent(req.body.returnTo);
    } else {
      req.session.returnTo = '/';
    }
    next();
  }
  // authenticates
  var localAuthenticate = _passport2.default.authenticate(passportKey);
  var localAuthenticateAndRedirect = _passport2.default.authenticate(passportKey, {
    successReturnToOrRedirect: true,
    successRedirect: '/',
    failureRedirect: '/' + endpoint,
    failureFlash: true // allow flash messages
  });
  // api
  app.post(routePath + '/' + endpoint, returnTo, localAuthenticateAndRedirect);
  // if some server problem causes user to try to
  // reach these endpoints via GET requests from browser
  app.get(routePath + '/' + endpoint, function (req, res) {
    return res.redirect('/');
  });
  // logout
  app.post(routePath + '/logout', function (req, res) {
    if (req.user) {
      req.logout();
    }
    res.redirect('/' + endpoint);
  });
  // return
  return { localAuthenticate: localAuthenticate,
    localAuthenticateAndRedirect: localAuthenticateAndRedirect,
    returnTo: returnTo
  };
}